{ config, inputs, lib, pkgs, ... }:

let cfg = config.modules.desktop.gnome;
in {
  options.modules.desktop.gnome = {
    enable = lib.mkOption {
      default = false;
      example = true;
    };
  };

  config = lib.mkIf cfg.enable {
    services.xserver = {
      enable = true;
      displayManager.gdm.enable = true;
      desktopManager.gnome.enable = true;
    };

    services.flatpak.enable = true;
    services.fwupd.enable = true;

    programs.dconf.enable = true;
    dconf.settings = {
      "io/github/celluloid-player/celluloid" = {
        draggable-video-area-enable = true;
      };
      "org/gnome/desktop/interface" = {
        color-scheme = "prefer-dark";
        cursor-theme = "Adwaita";
        enable-hot-corners = false;
        font-name = "Cantarell 11";
        gtk-theme = "adw-gtk3-dark";
        icon-theme = "MoreWaita";
        monospace-font-name = "UbuntuMono Nerd Font 11";
        toolbar-style = "both-horiz";
      };
      "org/gnome/desktop/peripherals/touchpad" = {
        tap-to-click = true;
      };
      "org/gnome/desktop/sound" = {
        theme-name = "freedesktop";
      };
      "org/gnome/desktop/search-providers" = {
        disabled = [ "org.gnome.Epiphany.desktop" ];
      };
      "org/gnome/desktop/wm/keybindings" = {
        switch-group = [ "<Super>grave" ];
        switch-group-backward = [ "<Shift><Super>grave" ];
      };
      "org/gnome/desktop/wm/preferences" = {
        button-layout = "appmenu:close";
      };
      "org/gnome/gnome-session" = {
        auto-save-session = true;
      };
      "org/gnome/gnome-system-monitor" = {
        show-dependencies = true;
      };
      "org/gnome/mutter" = {
        center-new-windows = true;
        edge-tiling = true;
        experimental-features = [ "scale-monitor-framebuffer" ];
      };
      "org/gnome/settings-daemon/plugins/media-keys" = {
        volume-up = [
          "<Shift>F12"
          "XF86AudioRaiseVolume"
        ];
        volume-down = [
          "<Shift>F11"
          "XF86AudioLowerVolume"
        ];
      };
      "org/gnome/shell" = {
        disable-user-extensions = false;
        enabled-extensions = [
          # "another-window-session-manager@gmail.com"
          "blur-my-shell@aunetx"
          "burn-my-windows@schneegans.github.com"
          "CoverflowAltTab@palatis.blogspot.com"
          # "desktop-cube@schneegans.github.com"
          # "desktop-zoom@colin.kinlo.ch"
          "espresso@coadmunkee.github.com"
          # "flypie@schneegans.github.com"
          # "forge@jmmaranan.com"
          "hue-lights@chlumskyvaclav@gmail.com"
          "just-perfection-desktop@just-perfection"
          # "pano@elhan.io"
          # "paperwm@hedning:matrix.org"
          # "search-light@icedman.github.com"
          "space-bar@luchrioh"
          # "smart-auto-move@khimaros.com"
          # "systemd-manager@hardpixel.eu"
          # "tailscale-status@maxgallup.github.com"
          # "tiling-assistant@leleat-on-github"
          "Vitals@CoreCoding.com"
          # "wsmatrix@martin.zurowietz.de"
        ];
        favorite-apps = [
          "firefox.desktop"
          "org.gnome.Nautilus.desktop"
        ];
      };
      "org/gnome/shell/extensions/another-window-session-manager" = {
        enable-autorestore-sessions = true;
      };
      "org/gnome/shell/extensions/blur-my-shell/panel" = {
        static-blur = true;
      };
      "org/gnome/shell/extensions/blur-my-shell/applications" = {
        blur = false;
      };
      "org/gnome/shell/extensions/burn-my-windows" = {
        fire-close-effect = false;
        glide-open-effect = true;
        glide-close-effect = true;
      };
      "org/gnome/shell/extensions/desktop-zoom" = {
        mag-factor-delta = 0.07;
      };
      "org/gnome/shell/extensions/espresso" = {
        enable-fullscreen = true;
        show-indicator = true;
        show-notifications = false;
        inhibit-apps = [
          "com.obsproject.Studio.desktop"
        ];
      };
      "org/gnome/shell/extensions/paperwm" = {
        use-default-background = true;
      };
      "org/gnome/shell/extensions/forge" = {
        window-gap-size = 8;
        window-gap-hidden-on-single = false;
      };
      "org/gnome/shell/extensions/just-perfection" = {
        activities-button = false;
        window-demands-attention-focus = true;
        workspace-wrap-around = true;
      };
      "org/gnome/shell/extensions/space-bar/behavior" = {
        enable-activate-workspace-shortcuts = true;
        show-empty-workspaces = true;
        smart-workspace-names = false;
      };
      "org/gnome/shell/extensions/tiling-assistant" = {
        screen-top-gap = 8;
        screen-right-gap = 8;
        screen-bottom-gap = 8;
        screen-left-gap = 8;
        window-gap = 8;
      };
      "org/gnome/Console" = {
        font-scale = 1.4;
        use-system-font = false;
        custom-font = "ComicShannsMono Nerd Font 10";
      };
      "org/gtk/settings/file-chooser" = {
        show-hidden = true;
        sort-directories-first = true;
      };
      "org/gtk/gtk4/settings/file-chooser" = {
        show-hidden = true;
        sort-directories-first = true;
      };
    };

    environment.sessionVariables = {
      QT_STYLE_OVERRIDE = lib.mkForce "kvantum";
      QT_WAYLAND_DECORATION = lib.mkForce "adwaita";
    };

    home.configFile = {
      "Kvantum/kvantum.kvconfig".text = lib.generators.toINI {} {
        General.theme = "KvLibadwaitaDark";
      };
      "Kvantum/KvLibadwaita".source = "${inputs.kvlibadwaita}/src/KvLibadwaita";
    };

    user.packages = with pkgs; [
      authenticator
      # bottles
      # bustle
      celluloid
      # d-spy
      # drawing
      # fragments
      gnome.ghex
      # gnome-builder
      gnome-decoder
      gnome-firmware
      gnome-frog
      # gnome-obfuscate
      gnome-podcasts
      identity
      mission-center
      mousam
      newsflash
      # schemes
      shortwave
      sysprof
    ];

    environment.systemPackages = with pkgs; [
      adw-gtk3
      gnomeExtensions.another-window-session-manager
      # gnomeExtensions.bifocals
      gnomeExtensions.blur-my-shell
      gnomeExtensions.browser-tabs
      gnomeExtensions.burn-my-windows
      gnomeExtensions.desktop-cube
      # gnomeExtensions.desktop-zoom
      gnome44Extensions."flypie@schneegans.github.com"
      # gnomeExtensions.forge
      # gnomeExtensions.gsnap
      gnomeExtensions.hue-lights
      gnomeExtensions.just-perfection
      # gnomeExtensions.mutter-primary-gpu
      gnomeExtensions.pano
      gnomeExtensions.paperwm
      # gnomeExtensions.pip-on-top
      gnomeExtensions.rounded-window-corners
      gnomeExtensions.search-light
      gnomeExtensions.smart-auto-move
      gnomeExtensions.space-bar
      gnomeExtensions.systemd-manager
      gnomeExtensions.tailscale-status
      gnomeExtensions.tiling-assistant
      # gnomeExtensions.todotxt
      gnomeExtensions.vitals
      # gnomeExtensions.window-is-ready-remover
      # gnomeExtensions.worksets
      # gnomeExtensions.workspace-matrix
      unstable.gnomeExtensions.coverflow-alt-tab
      unstable.gnomeExtensions.espresso
      unstable.kdePackages.qtstyleplugin-kvantum
      unstable.libsForQt5.qtstyleplugin-kvantum
      unstable.morewaita-icon-theme
      unstable.qadwaitadecorations
      unstable.qadwaitadecorations-qt6
    ] ++ (if config.virtualisation.podman.enable then [
      pods
    ] else []);

    home.services.gpg-agent.pinentryPackage = pkgs.pinentry-gnome3;
  };
}
